@@ -1,10 +1,13 @@
 /*
- * Copyright 2009-2011 JRuby Team (www.jruby.org).
+ * Copyright 2009-2012 JRuby Team (www.jruby.org).
  */
 
 #include <cstring>
 #include <cstdlib>
+#include <climits>
 #include <memory>
+#include <string>
+#include <unistd.h>
 #include "utilsfuncs.h"
 #include "argparser.h"
 #include "argnames.h"
@@ -28,7 +31,6 @@ Options:\n\
   -Xversion             print launcher's version\n\
 \nJvm Management:\n\
   -Xjdkhome <path>      set path to JDK\n\
-  -Xfork-java           run java in separate process\n\
   -J<jvm_option>        pass <jvm_option> to JVM\n\
 \nClasspath Management:\n\
   -Xcp   <classpath>    set the classpath\n\
@@ -41,7 +43,9 @@ Options:\n\
   -Xprop.erty[=value]   equivalent to -J-Djruby.<prop.erty>[=value]\n\
   -Xproperties          list supported properties (omit \"jruby.\" with -X)\n"
 #ifdef WIN32
-"  -Xconsole <mode>      jrubyw console attach mode (new|attach|suppress)\n"
+"\
+  -Xfork-java           run java in separate process\n\
+  -Xconsole <mode>      jrubyw console attach mode (new|attach|suppress)\n"
 #endif
 "\n"
 ;
@@ -52,6 +56,7 @@ const char *ArgParser::OPT_JDK_HOME = "-Djdk.home=";
 const char *ArgParser::OPT_JRUBY_HOME = "-Djruby.home=";
 const char *ArgParser::OPT_JRUBY_COMMAND_NAME = "-Dsun.java.command=";
 
+const char *ArgParser::OPT_CMDLINE_CLASS_PATH = "-cp";
 const char *ArgParser::OPT_CLASS_PATH = "-Djava.class.path=";
 const char *ArgParser::OPT_BOOT_CLASS_PATH = "-Xbootclasspath/a:";
 
@@ -62,7 +67,7 @@ const char *ArgParser::MAIN_CLASS = "org/jruby/Main";
 const char *ArgParser::DEFAULT_EXECUTABLE = "jruby";
 
 ArgParser::ArgParser()
-    : separateProcess(false)
+    : separateProcess(true)
     , nailgunClient(false)
     , noBootClassPath(false)
     , printCommandLine(false)
@@ -113,21 +118,42 @@ void ArgParser::addEnvVarToOptions(std::list<std::string> & optionsList, const c
 #include <mach-o/dyld.h>
 #endif
 
-bool ArgParser::initPlatformDir() {
-#ifdef WIN32
-    char path[MAX_PATH] = "";
-    getCurrentModulePath(path, MAX_PATH);
-#else
-    char path[PATH_MAX] = "";
-    bool found = false;
+#ifndef PATH_MAX
+#define PATH_MAX  MAX_PATH
+#endif
 
-    // first try via linux /proc/self/exe
-    logMsg("initPlatformDir: trying /proc/self/exe");
-    found = readlink("/proc/self/exe", path, PATH_MAX) != -1;
+bool ArgParser::initPlatformDir() {
+    bool found = false;
+    char path[PATH_MAX] = "";
+
+    if (getenv("JRUBY_HOME") != NULL) {
+        logMsg("initPlatformDir: using JRUBY_HOME environment variable");
+        char sep[2] = { FILE_SEP, NULL };
+        strncpy(path, getenv("JRUBY_HOME"), PATH_MAX - 11);
+        strncpy(path + strlen(path), sep, 1);
+        strncpy(path + strlen(path), "bin", 3);
+        strncpy(path + strlen(path), sep, 1);
+        strncpy(path + strlen(path), "jruby", 5);
+        found = true;
+    }
+
+#ifdef WIN32
+
+    if (!found) {
+        getCurrentModulePath(path, PATH_MAX);
+    }
+
+#else // !WIN32
+
+    if (!found) {
+        // first try via linux /proc/self/exe
+        logMsg("initPlatformDir: trying /proc/self/exe");
+        found = readlink("/proc/self/exe", path, PATH_MAX) != -1;
+    }
 
 #ifdef __MACH__
     uint32_t sz = PATH_MAX;
-    if (_NSGetExecutablePath(path, &sz) == 0) { // OSX-specific
+    if (!found && _NSGetExecutablePath(path, &sz) == 0) { // OSX-specific
         logMsg("initPlatformDir: using _NSGetExecutablePath");
         string tmpPath(path);
         realpath(tmpPath.c_str(), path);
@@ -137,7 +163,7 @@ bool ArgParser::initPlatformDir() {
 
 #ifdef __SUNOS__
     const char* execname = getexecname();
-    if (execname) {
+    if (!found && execname) {
         logMsg("initPlatformDir: using getexecname");
         char * dst = path;
         if (execname[0] != '/') {
@@ -163,32 +189,19 @@ bool ArgParser::initPlatformDir() {
         strncpy(path + strlen(path), platformDir.c_str(), platformDir.length());
         found = true;
     }
+#endif // WIN32
 
     if (!found) {               // try via PATH search
         logMsg("initPlatformDir: trying to find executable on PATH");
-        char * location = findOnPath(platformDir.c_str());
-        if (location != NULL) {
-            strncpy(path, location, PATH_MAX);
-            free(location);
+        std::string location = findOnPath(platformDir.c_str());
+        if (location.size() > 0) {
+            strncpy(path, location.c_str(), PATH_MAX);
             found = true;
         }
     }
 
-    if (!found) {               // try via JRUBY_HOME
-        if (getenv("JRUBY_HOME") != NULL) {
-            logMsg("initPlatformDir: trying JRUBY_HOME environment variable");
-            strncpy(path, getenv("JRUBY_HOME"), PATH_MAX - 11);
-            strncpy(path + strlen(path), "/bin/jruby", 10);
-            found = true;
-        }
-    }
-
-    if (!fileExists(path)) {
-        printToConsole("Could not figure out a proper location for JRuby.\n"
-                       "Try `jruby -Xtrace trace.log ...` and view trace.log for details.");
-        return false;
-    }
-#endif
+    // Check if bin/jruby file exists; this logs a message if not found
+    fileExists(path);
 
     logMsg("Module: %s", path);
     char *bslash = strrchr(path, FILE_SEP);
@@ -230,6 +243,16 @@ bool ArgParser::parseArgs(int argc, char *argv[]) {
         noBootClassPath = true;
     }
 
+    const char *java_mem = getenv("JAVA_MEM");
+    if (java_mem != NULL) {
+        javaOptions.push_back(java_mem);
+    }
+
+    const char *java_stack = getenv("JAVA_STACK");
+    if (java_stack != NULL) {
+        javaOptions.push_back(java_stack);
+    }
+
     addToArgList(args, argc, argv);
 
     logMsg("Parsing arguments:");
@@ -343,6 +366,9 @@ bool ArgParser::parseArgs(int argc, char *argv[]) {
 }
 
 void ArgParser::prepareOptions() {
+    list<string> userOptions(javaOptions);
+    javaOptions.clear();
+
     string option = OPT_JDK_HOME;
     option += jdkhome;
     javaOptions.push_back(option);
@@ -390,7 +416,7 @@ void ArgParser::prepareOptions() {
 #endif
     javaOptions.push_back(option);
 
-    setupMaxHeapAndStack();
+    setupMaxHeapAndStack(userOptions);
 
     constructBootClassPath();
     constructClassPath();
@@ -411,22 +437,30 @@ void ArgParser::prepareOptions() {
     option += cmdName;
     javaOptions.push_back(option);
 
-    option = OPT_CLASS_PATH;
-    option += classPath;
-    javaOptions.push_back(option);
+    // When launching a separate process, use '-cp' which expands embedded wildcards
+    if (separateProcess) {
+        javaOptions.push_back(OPT_CMDLINE_CLASS_PATH);
+        javaOptions.push_back(classPath);
+    } else {
+        option = OPT_CLASS_PATH;
+        option += classPath;
+        javaOptions.push_back(option);
+    }
 
     if (!bootClassPath.empty()) {
         option = OPT_BOOT_CLASS_PATH;
         option += bootClassPath;
         javaOptions.push_back(option);
     }
+
+    javaOptions.insert(javaOptions.end(), userOptions.begin(), userOptions.end());
 }
 
-void ArgParser::setupMaxHeapAndStack() {
+void ArgParser::setupMaxHeapAndStack(list<string> userOptions) {
     // Hard-coded 500m, 2048k is for consistency with jruby shell script.
     string heapSize("500m"), stackSize("2048k");
     bool maxHeap = false, maxStack = false;
-    for (list<string>::iterator it = javaOptions.begin(); it != javaOptions.end(); it++) {
+    for (list<string>::iterator it = userOptions.begin(); it != userOptions.end(); it++) {
         if (!maxHeap && strncmp("-Xmx", it->c_str(), 4) == 0) {
             heapSize = it->substr(4, it->size() - 4);
             maxHeap = true;
