@@ -1,4 +1,5 @@
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include "unixlauncher.h"
 #include "utilsfuncs.h"
@@ -29,8 +30,15 @@ int UnixLauncher::run(int argc, char* argv[], char* envp[]) {
     if (nailgunClient) {
         progArgs.push_front("org.jruby.util.NailMain");
         char ** nailArgv = convertToArgvArray(progArgs);
+        int nailArgc = progArgs.size();
+
         if (printCommandLine) {
-            return printArgvToConsole(nailArgv);
+            printListToConsole(progArgs);
+            for (int i = 0; i < nailArgc; i++) {
+                free(nailArgv[i]);
+            }
+            delete[] nailArgv;
+            return 0;
         }
         return nailgunClientMain(progArgs.size(), (char**)nailArgv, envp);
     }
@@ -83,7 +91,9 @@ int UnixLauncher::run(int argc, char* argv[], char* envp[]) {
         if (!jdkhome.empty()) {
             java = jdkhome + "/bin/java";
         } else if (getenv("JAVA_HOME") != NULL) {
-            java = string(getenv("JAVA_HOME")) + "/bin/java";
+            string java_home = string(getenv("JAVA_HOME"));
+            java_home = trimTrailingBackslashes(java_home);
+            java = java_home + "/bin/java";
         } else {
             java = findOnPath("java");
         }
@@ -104,12 +114,17 @@ int UnixLauncher::run(int argc, char* argv[], char* envp[]) {
     }
 
     char** newArgv = convertToArgvArray(commandLine);
+    int newArgc = commandLine.size();
 
     if (printCommandLine) {
-        return printArgvToConsole(newArgv);
+        printListToConsole(commandLine);
+        for (int i = 0; i < newArgc; i++) {
+            free(newArgv[i]);
+        }
+        delete[] newArgv;
+        return 0;
     }
 
-
     if (!fileExists(java.c_str())) {
         string msg = "No `java' exists at " + java + ", please double-check JAVA_HOME.\n";
         printToConsole(msg.c_str());
